package com.wtkj.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.wtkj.entity.Task;
import com.wtkj.service.IIndexService;
import com.wtkj.service.IWorkStatusService;
import com.wtkj.wrapper.IndexTaskScheduleWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

import static com.wtkj.config.MagicValue.*;

/**
 * @author Blizzard
 * @create at 2023-09-15 23:58
 * @describe
 */
@RestController
@RequestMapping("/index")
@Api(value = "首页", tags = "首页")
@AllArgsConstructor
public class IndexController {

	private final IWorkStatusService workStatus;
	private final IIndexService indexService;

	/**
	 * 用户工作状态
	 */
	@GetMapping("/work-status")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "我的状态", notes = "")
	public R wordStatus() {
		return R.data(workStatus.workStatus());
	}


	/**
	 * 首页统计
	 */
	@GetMapping("/project-and-task")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "项目和任务统计", notes = "")
	public R projectAndTask(@RequestParam Long topDept) {
		return R.data(indexService.projectAndTaskSummary(topDept));
	}

	/**
	 * 待办事项
	 */
	@GetMapping("/todo-list")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "待办事项", notes = "传入isRead查看状态 0:全部 1:已读 2: 未读 默认未读,分页参数")
	public R todoList(@RequestParam(defaultValue = "2") Integer isRead, Query query) {
		return R.data(indexService.todoList(isRead, Condition.getPage(query)));
	}

	/**
	 * 学习资料
	 */
	@GetMapping("/study-list")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "学习资料", notes = "")
	public R noticeList(Query query) {
		return R.data(indexService.studyList(Condition.getPage(query)));
	}

	/**
	 * 阅读消息
	 */
	@GetMapping("/message-detail")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "阅读消息", notes = "传入id")
	public R messageDetail(@RequestParam Long id) {
		return R.data(indexService.messageDetail(id));
	}

	/**
	 * 工作日程
	 */
	@GetMapping("/user-schedule")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "用户工作日程", notes = "传入登录用户的顶级机构id,日期字符串")
	public R schedule(@RequestParam Long topDept, String date) {
		return R.data(indexService.schedule(topDept, date));
	}

	/**
	 * 工作日程展开
	 */
	@GetMapping("/user-schedule-expand")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "用户工作日程展开", notes = "传入用户id，日期字符串")
	public R scheduleExpand(@RequestParam Long userId, String date) {
		List<Task> taskList = indexService.scheduleExpand(userId, date);
		return R.data(IndexTaskScheduleWrapper.build().listVO(taskList));
	}


	/**
	 * 任务分布
	 */
	@GetMapping("/task-statistics")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "任务分布统计", notes = "传入日期")
	public R taskTaskStatistics(@RequestParam Long topDept, @RequestParam String date) {
		String userRole = AuthUtil.getUserRole();
		if (userRole != null) {
			if (userRole.contains(STAFF_ADMIN)) {
				return R.data(indexService.mapStatistics(topDept, date));
			} else if (userRole.contains(STAFF_SECOND_ADMIN)) {
				return R.data(indexService.myStatistics(date));
			} else if (userRole.contains(STAFF_USER)) {
				return R.data(null);
			}
		}
		return R.fail("获取用户角色失败");
	}

}
