package com.wtkj.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.wtkj.dto.InviteToDeptDTO;
import com.wtkj.service.IQrcodeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springframework.web.bind.annotation.*;

/**
 * @author Blizzard
 * @create at 2023-09-19 17:25
 * @describe
 */
@RestController
@AllArgsConstructor
@RequestMapping("/qrcode")
@Api(value = "二维码模块", tags = "二维码模块")
public class QrCodeController {

	private final IQrcodeService qrcodeService;

	/**
	 * 生成二维码
	 */
	@GetMapping("/create-qrcode")
	@ApiOperation(value = "生成二维码", notes = "")
	@ApiOperationSupport(order = 1)
	public R createQrcode() {
		return R.data(qrcodeService.createQrcode());
	}

	/**
	 * 邀请至机构下
	 */
	@GetMapping("/invite-to-dept")
	@ApiOperation(value = "接受邀请至机构下", notes = "type:1 邀请内部联系人 2:外部联系人")
	@ApiOperationSupport(order = 2)
	public R inviteToDept(@RequestBody InviteToDeptDTO dto) {
		return R.status(qrcodeService.inviteToDept(dto.getCreateUser(), dto.getType(), dto.getQrcodeId(), AuthUtil.getUserId(), dto.getDeptId()));
	}

	/**
	 * 邀请至项目组
	 */
	@GetMapping("/invite-to-project")
	@ApiOperation(value = "接受邀请至项目组", notes = "ownerDept:主管单位，deptId:服务商的顶级机构id")
	@ApiOperationSupport(order = 3)
	public R inviteToProject(@RequestParam String qrcodeId, @RequestParam Long ownerDept, @RequestParam Long deptId, @RequestParam Long projectId) {
		return R.status(qrcodeService.inviteToProject(qrcodeId, ownerDept, deptId, projectId));
	}

}
