package com.wtkj.service.impl;

import com.wtkj.entity.*;
import com.wtkj.service.*;
import lombok.AllArgsConstructor;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.system.entity.Dept;
import org.springblade.system.feign.ISysClient;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

import static com.wtkj.config.MagicValue.ZERO;

/**
 * @author Blizzard
 * @create at 2023-09-13 16:32
 * @describe
 */
@Service
@AllArgsConstructor
public class AsyncServiceImpl implements IAsyncService {

	private final ITypeAndStageService typeAndStageService;
	private final IProjectStageService stageService;
	private final IProjectAuthService projectAuthService;
	private final ISysClient sysClient;
	private final IUserClient userClient;
	private final ITaskLogService taskLogService;
	private final IIndexMessageService indexMessageService;

	@Override
	@Async("asyncPoolTaskExecutor")
	public void createProject(Project project, BladeUser user) {
		//1.项目类型  --> 使用类型对应的模版阶段创建项目阶段
		Long projectType = project.getProjectType();
		List<TemplateTypeAndStage> templateStage = typeAndStageService.getStages(projectType);
		//2.创建项目阶段
		if (!CollectionUtils.isEmpty(templateStage)) {
			List<ProjectStage> stagesList = new ArrayList<>();
			templateStage.forEach(template -> {
				//项目的阶段记录表
				ProjectStage stage = new ProjectStage();
				stage.setProjectId(project.getId());
				stage.setName(template.getName());
				stage.setSort(template.getSort());
				stage.setCreateUser(project.getCreateUser());
				stage.setCreateDept(project.getCreateDept());
				stage.setCreateTime(DateUtil.now());
				stage.setUpdateTime(DateUtil.now());
				stage.setUpdateUser(project.getUpdateUser());
				stagesList.add(stage);
			});
			if (!CollectionUtils.isEmpty(stagesList)) {
				stageService.saveBatch(stagesList);
			}
		}

		//3.如果主管单位和当前登录用户不是同一机构
		String deptIdStr = user.getDeptId();
		if (deptIdStr != null) {
			Long deptId = Func.firstLong(deptIdStr);
			if (!deptId.equals(project.getCompetentUnit())) {
				ProjectAuth auth = new ProjectAuth();
				auth.setUserId(user.getUserId());
				R<Dept> rpc = sysClient.getDept(deptId);
				if (rpc.isSuccess()) {
					String ancestors = rpc.getData().getAncestors();
					if (ancestors != null && ancestors.contains(String.valueOf(ZERO))) {
						List<Long> longList = Func.toLongList(ancestors);
						if (!CollectionUtils.isEmpty(longList)) {
							if (longList.size() > 2) {
								auth.setTopDept(longList.get(1));
							} else {
								auth.setTopDept(Long.valueOf(deptIdStr));
							}
						}
					}
				}
				auth.setUserDept(deptId);
				auth.setProjectId(project.getId());
				projectAuthService.save(auth);
			}
		}

	}

	@Async("asyncPoolTaskExecutor")
	@Override
	public void removeProject(List<Long> longList) {

	}

	@Override
	@Async("asyncPoolTaskExecutor")
	public void createTaskLog(Task task) {
		//1.任务日志
		Long createUser = task.getCreateUser();
		if (createUser != null) {
			R<User> userR = userClient.userInfoById(createUser);
			if (userR.isSuccess()) {
				List<TaskLog> logs = new ArrayList<>();
				TaskLog log = new TaskLog();
				log.setTaskId(task.getId());
				log.setContent(userR.getData().getName() + " 创建了任务");
				log.setCreateDept(task.getCreateDept());
				log.setCreateUser(task.getCreateUser());
				log.setUpdateUser(task.getUpdateUser());
				log.setCreateTime(DateUtil.now());
				log.setUpdateTime(DateUtil.now());
				logs.add(log);
				if (task.getOrgDeptId() != null) {
					Long orgDeptId = task.getOrgDeptId();
					R<Dept> dept = sysClient.getDept(orgDeptId);
					if (dept.isSuccess()) {
						TaskLog taskLog = new TaskLog();
						taskLog.setTaskId(task.getId());
						taskLog.setContent(userR.getData().getName() + " 指定了执行单位为" + dept.getData().getDeptName());
						taskLog.setCreateDept(task.getCreateDept());
						taskLog.setCreateUser(task.getCreateUser());
						taskLog.setUpdateUser(task.getUpdateUser());
						taskLog.setCreateTime(DateUtil.now());
						taskLog.setUpdateTime(DateUtil.now());
						logs.add(taskLog);
					}
				}
				if (task.getExecuteDept() != null) {
					Long executeDept = task.getExecuteDept();
					R<Dept> dept = sysClient.getDept(executeDept);
					if (dept.isSuccess()) {
						TaskLog taskLog = new TaskLog();
						taskLog.setTaskId(task.getId());
						taskLog.setContent(userR.getData().getName() + " 指定了任务执行部门为" + dept.getData().getDeptName());
						taskLog.setCreateDept(task.getCreateDept());
						taskLog.setCreateUser(task.getCreateUser());
						taskLog.setUpdateUser(task.getUpdateUser());
						taskLog.setCreateTime(DateUtil.now());
						taskLog.setUpdateTime(DateUtil.now());
						logs.add(taskLog);
					}
				}
				if (task.getProjectManager() != null) {
					Long projectManager = task.getProjectManager();
					R<User> userR1 = userClient.userInfoById(projectManager);
					if (userR1.isSuccess()) {
						TaskLog taskLog = new TaskLog();
						taskLog.setTaskId(task.getId());
						taskLog.setContent(userR.getData().getName() + " 指定了项目经理为 " + userR1.getData().getName());
						taskLog.setCreateDept(task.getCreateDept());
						taskLog.setCreateUser(task.getCreateUser());
						taskLog.setUpdateUser(task.getUpdateUser());
						taskLog.setCreateTime(DateUtil.now());
						taskLog.setUpdateTime(DateUtil.now());
						logs.add(taskLog);
					}
				}
				if (task.getExecuteUser() != null) {
					List<Long> userIds = Func.toLongList(task.getExecuteUser());
					List<String> str = new ArrayList<>();
					userIds.forEach(userId -> {
						R<User> rpc = userClient.userInfoById(userId);
						if (rpc.isSuccess()) {
							str.add(rpc.getData().getName());
						}
					});
					if (!CollectionUtils.isEmpty(str)) {
						TaskLog taskLog = new TaskLog();
						taskLog.setTaskId(task.getId());
						taskLog.setContent(userR.getData().getName() + " 指定了任务执行者 " + Func.join(str));
						taskLog.setCreateDept(task.getCreateDept());
						taskLog.setCreateUser(task.getCreateUser());
						taskLog.setUpdateUser(task.getUpdateUser());
						taskLog.setCreateTime(DateUtil.now());
						taskLog.setUpdateTime(DateUtil.now());
						logs.add(taskLog);
					}
				}
				taskLogService.saveBatch(logs);
			}
		}
		// todo 2.任务下发通知
	}

	@Override
	@Async("asyncPoolTaskExecutor")
	public void updateTask(Integer type, Long userId, Task task) {
		R<User> userR = userClient.userInfoById(userId);
		if (userR.isSuccess()) {
			List<TaskLog> logs = new ArrayList<>();
			switch (type) {
				case 1:
					Long orgDeptId = task.getOrgDeptId();
					R<Dept> dept = sysClient.getDept(orgDeptId);
					if (dept.isSuccess()) {
						TaskLog log = new TaskLog();
						log.setTaskId(task.getId());
						log.setContent(userR.getData().getName() + " 指定了执行单位为" + dept.getData().getDeptName());
						log.setCreateDept(task.getCreateDept());
						log.setCreateUser(task.getCreateUser());
						log.setUpdateUser(task.getUpdateUser());
						log.setCreateTime(DateUtil.now());
						log.setUpdateTime(DateUtil.now());
						logs.add(log);
					}
					break;
				case 2:
					Long executeDept = task.getExecuteDept();
					R<Dept> dept1 = sysClient.getDept(executeDept);
					if (dept1.isSuccess()) {
						TaskLog log1 = new TaskLog();
						log1.setTaskId(task.getId());
						log1.setContent(userR.getData().getName() + " 指定了任务执行部门为" + dept1.getData().getDeptName());
						log1.setCreateDept(task.getCreateDept());
						log1.setCreateUser(task.getCreateUser());
						log1.setUpdateUser(task.getUpdateUser());
						log1.setCreateTime(DateUtil.now());
						log1.setUpdateTime(DateUtil.now());
						logs.add(log1);
					}
					break;
				case 3:
					Long projectManager = task.getProjectManager();
					R<User> userR1 = userClient.userInfoById(projectManager);
					if (userR1.isSuccess()) {
						TaskLog log2 = new TaskLog();
						log2.setTaskId(task.getId());
						log2.setContent(userR.getData().getName() + " 指定了项目经理为 " + userR1.getData().getName());
						log2.setCreateDept(task.getCreateDept());
						log2.setCreateUser(task.getCreateUser());
						log2.setUpdateUser(task.getUpdateUser());
						log2.setCreateTime(DateUtil.now());
						log2.setUpdateTime(DateUtil.now());
						logs.add(log2);
					}
					break;
				case 4:
					List<Long> userIds = Func.toLongList(task.getExecuteUser());
					List<String> str = new ArrayList<>();
					userIds.forEach(id -> {
						R<User> rpc = userClient.userInfoById(id);
						if (rpc.isSuccess()) {
							str.add(rpc.getData().getName());
						}
					});
					if (!CollectionUtils.isEmpty(str)) {
						TaskLog log3 = new TaskLog();
						log3.setTaskId(task.getId());
						log3.setContent(userR.getData().getName() + " 指定了任务执行者 " + Func.join(str));
						log3.setCreateDept(task.getCreateDept());
						log3.setCreateUser(task.getCreateUser());
						log3.setUpdateUser(task.getUpdateUser());
						log3.setCreateTime(DateUtil.now());
						log3.setUpdateTime(DateUtil.now());
						logs.add(log3);
					}
					break;
			}

			taskLogService.saveBatch(logs);
		}
	}

	@Override
	@Async("asyncPoolTaskExecutor")
	public void submitTaskMessage(Long userId, Task task) {
		Long createUser = task.getCreateUser();
		Long projectId = task.getProjectId();
		String title = task.getTitle();
		R<User> userR = userClient.userInfoById(userId);
		if (userR.isSuccess()) {
			IndexMessage message = new IndexMessage();
			message.setCategory(1);
			//发送给审查人和项目经理
			message.setToUser(createUser);
			message.setContent(userR.getData().getName() + "上传了" + title + "任务的文件，请查阅");
			message.setOpenUrl("/task");
			message.setCreateDept(task.getCreateDept());
			message.setCreateUser(userId);
			message.setUpdateUser(userId);
			message.setCreateTime(DateUtil.now());
			message.setUpdateTime(DateUtil.now());
			indexMessageService.save(message);
		}
	}


}
