package com.wtkj.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.wtkj.entity.ContactOuter;
import com.wtkj.entity.ProjectAuth;
import com.wtkj.entity.ProjectGroup;
import com.wtkj.service.IContactOuterService;
import com.wtkj.service.IProjectAuthService;
import com.wtkj.service.IProjectGroupService;
import com.wtkj.service.IQrcodeService;
import io.seata.spring.annotation.GlobalTransactional;
import lombok.AllArgsConstructor;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.system.user.feign.IUserClient;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.Duration;

import static com.wtkj.config.NacosConfigValue.expireIn;
import static com.wtkj.config.NacosConfigValue.scanCount;

/**
 * @author Blizzard
 * @create at 2023-09-19 17:35
 * @describe
 */
@AllArgsConstructor
@Service
public class QrcodeServiceImpl implements IQrcodeService {

	private final BladeRedis redis;
	private final IUserClient userClient;
	private final IProjectAuthService projectAuthService;
	private final IProjectGroupService projectGroupService;
	private final IContactOuterService contactOuterService;

	@Override
	public JSONObject createQrcode() {
		JSONObject res = new JSONObject();
		String kv = IdWorker.getIdStr();
		Long userId = AuthUtil.getUserId();
		redis.setEx(kv, 0, Duration.ofSeconds(expireIn));
		res.put("qrcodeId", kv);
		res.put("createUser", userId);
		return res;
	}

	@Override
	@GlobalTransactional(rollbackFor = Exception.class)
	public boolean inviteToDept(Long createUser, Integer type, String qrcodeId, Long userId, Long deptId) {
		boolean flag = false;
		boolean exist = isExist(qrcodeId);
		//二维码可用前提下
		if (exist) {
			//1.绑定机构
			R<Boolean> rpc = userClient.bindUser(userId, deptId);
			//2.保存外部联系人
			if (rpc.isSuccess()) {
				flag = rpc.getData();
			}
			if (type == 2) {
				ContactOuter contact = new ContactOuter();
				contact.setRelatedDeptId(deptId);
				contact.setRelatedUserId(userId);
				contact.setUserId(createUser);
				contactOuterService.save(contact);
			}

		}
		return flag;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean inviteToProject(String qrcodeId, Long ownerDept, Long deptId, Long projectId) {
		boolean flag = false;
		boolean exist = isExist(qrcodeId);
		if (exist) {
			//1.权限 2.项目组
			ProjectAuth auth = new ProjectAuth();
			auth.setProjectId(projectId);
			auth.setTopDept(deptId);
			projectAuthService.save(auth);

			ProjectGroup group = new ProjectGroup();
			group.setProjectId(projectId);
			group.setInviteDept(ownerDept);
			group.setBeInvitedDept(deptId);
			flag = projectGroupService.save(group);
		}
		return flag;
	}

	/**
	 * 判断二维码是否有效和是否达到扫描上限
	 */
	private boolean isExist(String key) {
		boolean flag = false;
		Boolean exists = redis.exists(key);
		//如果存在
		if (exists) {
			Long count = redis.incrBy(key, 1);
			if (count > scanCount) {
				throw new ServiceException("二维码只能扫描" + scanCount + "次数");
			} else {
				flag = true;
			}
		}
		return flag;
	}
}
