package com.wtkj.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.wtkj.entity.Project;
import com.wtkj.entity.Task;
import com.wtkj.entity.TaskCost;
import com.wtkj.mapper.TaskCostMapper;
import com.wtkj.service.IProjectService;
import com.wtkj.service.ITaskCostService;
import com.wtkj.service.ITaskService;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDate;
import java.time.Period;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author Blizzard
 * @create at 2023-09-18 14:16
 * @describe
 */
@Service
@AllArgsConstructor
public class TaskCostServiceImpl extends BaseServiceImpl<TaskCostMapper, TaskCost> implements ITaskCostService {

	private final ITaskService taskService;
	private final IProjectService projectService;

	@Override
	public IPage<TaskCost> pageByTaskId(Long taskId, IPage<TaskCost> page) {
		return page.setRecords(baseMapper.selectByTaskId(taskId, page));
	}

	@Override
	public List<TaskCost> listByTaskId(Long taskId) {
		return baseMapper.listByTaskId(taskId);
	}

	@Override
	public JSONObject costStatistics(Long taskId) {
		JSONObject res = new JSONObject();
		int days = 0;
		BigDecimal amount = BigDecimal.valueOf(0);
		Task task = taskService.getById(taskId);
		if (task != null) {
			Long projectId = task.getProjectId();
			if (projectId != null) {
				Project byId = projectService.getById(projectId);
				if (byId != null) {
					Date createTime = byId.getCreateTime();
					if (createTime != null) {
						Instant instant = createTime.toInstant();
						ZoneId zoneId = ZoneId.systemDefault();
						LocalDate createDate = instant.atZone(zoneId).toLocalDate();
						LocalDate now = LocalDate.now(zoneId);
						Period between = Period.between(createDate, now);
						days = between.getDays();
					}
				}
			}
		}
		List<TaskCost> costs = this.listByTaskId(taskId);
		if (!CollectionUtils.isEmpty(costs)) {
			amount = costs.stream().map(TaskCost::getAmount).filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add);
		}
		res.put("days", days);
		res.put("amount", amount);
		return res;
	}
}
