package com.wtkj.wrapper;

import com.wtkj.entity.FileAndFolder;
import com.wtkj.entity.Project;
import com.wtkj.entity.Task;
import com.wtkj.service.IFileAndFolderService;
import com.wtkj.service.IProjectService;
import com.wtkj.vo.TaskVO;
import org.springblade.core.mp.support.BaseEntityWrapper;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.system.entity.Dept;
import org.springblade.system.entity.DictBiz;
import org.springblade.system.feign.IDictBizClient;
import org.springblade.system.feign.ISysClient;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author Blizzard
 * @create at 2023-09-15 16:18
 * @describe
 */
public class TaskWrapper extends BaseEntityWrapper<Task, TaskVO> {

	private static IUserClient userClient;
	private static ISysClient sysClient;
	private static IFileAndFolderService fileAndFolderService;
	private static IDictBizClient dictBizClient;

	private static IProjectService projectService;


	public static TaskWrapper build() {
		userClient = SpringUtil.getBean(IUserClient.class);
		sysClient = SpringUtil.getBean(ISysClient.class);
		fileAndFolderService = SpringUtil.getBean(IFileAndFolderService.class);
		dictBizClient = SpringUtil.getBean(IDictBizClient.class);
		projectService = SpringUtil.getBean(IProjectService.class);
		return new TaskWrapper();
	}

	@Override
	public TaskVO entityVO(Task entity) {
		TaskVO vo = Objects.requireNonNull(BeanUtil.copy(entity, TaskVO.class));
		Long projectId = entity.getProjectId();
		if (projectId != null) {
			Project byId = projectService.getById(projectId);
			if (byId != null) {
				vo.setProjectName(byId.getName());
			}
		}
		//联系人
		Long projectManager = vo.getProjectManager();
		if (projectManager != null) {
			R<User> userR = userClient.userInfoById(projectManager);
			if (userR.isSuccess()) {
				vo.setProjectManagerUser(userR.getData());
			}
		}
		//协作单位
		Long orgDeptId = vo.getOrgDeptId();
		if (orgDeptId != null) {
			R<Dept> rpc = sysClient.getDept(orgDeptId);
			if (rpc.isSuccess()) {
				vo.setDispatchUintName(rpc.getData().getDeptName());
			}
		}
		//执行机构
		Long executeDept = vo.getExecuteDept();
		if (executeDept != null) {
			R<Dept> dept = sysClient.getDept(executeDept);
			if (dept.isSuccess()) {
				vo.setExecuteDeptName(dept.getData().getDeptName());
			}
		}

		//任务创建人
		Long createUser = entity.getCreateUser();
		R<User> rpc = userClient.userInfoById(createUser);
		if (rpc.isSuccess()) {
			vo.setCreateUserName(rpc.getData().getName());
		}

		//审核人
		String checkUser = entity.getCheckUser();
		if (checkUser != null) {
			List<User> users = new ArrayList<>();
			List<Long> userIds = Func.toLongList(checkUser);
			userIds.forEach(userId -> {
				R<User> userR = userClient.userInfoById(userId);
				if (userR.isSuccess()) {
					users.add(userR.getData());
				}
			});
			vo.setCheckUsers(users);
		}

		//任务执行人
		String executeUser = entity.getExecuteUser();
		if (executeUser != null) {
			List<User> users = new ArrayList<>();
			List<Long> userIds = Func.toLongList(executeUser);
			userIds.forEach(userId -> {
				R<User> userR = userClient.userInfoById(userId);
				if (userR.isSuccess()) {
					users.add(userR.getData());
				}
			});
			vo.setExecuteUsers(users);
		}

		//任务相关联文件
		String relatedIds = entity.getRelatedIds();
		if (StringUtil.isNotBlank(relatedIds)) {
			List<FileAndFolder> files = new ArrayList<>();
			List<Long> fileIds = Func.toLongList(relatedIds);
			fileIds.forEach(fileId -> {
				FileAndFolder byId = fileAndFolderService.getById(fileId);
				if (byId != null) {
					files.add(byId);
				}
			});
			vo.setFiles(files);
		}
		//任务标签的数组
		String tags = entity.getTags();
		String category = entity.getCategory();
		List<String> dictBizs = new ArrayList<>();
		if (StringUtil.isNotBlank(tags)) {
			List<Long> intList = Func.toLongList(tags);
			intList.forEach(f -> {
				R<DictBiz> byId = dictBizClient.getById(f);
				if (byId.isSuccess()) {
					dictBizs.add(byId.getData().getDictValue());
				}
			});
		}
		if (StringUtil.isNotBlank(category)) {
			List<Long> intList = Func.toLongList(category);
			intList.forEach(f -> {
				R<DictBiz> byId = dictBizClient.getById(f);
				if (byId.isSuccess()) {
					dictBizs.add(byId.getData().getDictValue());
				}
			});
		}
		vo.setTagsArray(dictBizs);
		return vo;
	}
}
